#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. module:: NodeosConnect
   :synopsis: This Class creates connection to EOS nodeos API RPC and the Wallet API RPC.
              Only to methods are implemented GET and POST.
.. author: Merouane Benthameur <merouane.benth@gmail.com>
"""

import requests

__version: "0.1.2"


class RequestHandlerAPI:
    """ a class to handle the http connection with the EOS node."""

    def __init__(self, base_url,  verify=False, **kwargs):
        self.base_url = base_url
        self.session = requests.Session()
        self.ssl_verify = verify

        for arg in kwargs:
            if isinstance(kwargs[arg], dict):
                kwargs[arg] = self.__set_session_attr(
                    getattr(self.session, arg), kwargs[arg])
            setattr(self.session, arg, kwargs[arg])

    def get(self, path, **kwargs):
        """
        A GET Http method.

        :param path: str: path to  api endpoint
        :param kwargs: json: arguments auth, headers, data ..etc.

        :return: response object

        """
        try:
            return self.session.get(self.base_url + path, verify=self.ssl_verify, **kwargs)
        except requests.exceptions.RequestException as e:
            raise e

    def post(self, path, **kwargs):
        """
        A POST Http method.

        :param path: str: path to  api endpoint
        :param kwargs: json: arguments auth, headers, data ..etc.

        :return: response object

        """
        try:
            return self.session.post(self.base_url + path, verify=self.ssl_verify, **kwargs)
        except requests.exceptions.RequestException as e:
            raise e

    @staticmethod
    def __set_session_attr(source, destination):
        """
        set session attributes
        :param source: the default attributes when starting the session
        :param destination: json: target attributes to be set

        :return:json: session attributes

        """
        for key, value in source.items():
            if isinstance(value, dict):
                node = destination.setdefault(key, {})
                RequestHandlerAPI.__set_session_attr(value, node)
            else:
                destination[key] = value
        return destination



__all__ = ['NodeosConnect', 'EOSChainApi', 'EOSWalletApi']

# deprecated to keep older scripts who import this from breaking
from scripts.NodeosConnect import RequestHandlerAPI
from scripts.EOSChainApi import ChainAPI
from scripts.EOSWalletApi import WalletAPI



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. module:: EOSChainApi
   :synopsis: A basic wrapper around the EOS Chain API..
.. author:: Merouane Benthameur <merouane.benth@gmail.com>
"""


class ChainAPI:
    """ wrapper for EOS Chain API"""

    def __init__(self, connection_session):
        """
        constructor  of the ChainAPI
        :param connection_session: session request object.
        """
        self.session = connection_session

    def get_info(self):
        """
        Get latest information related to a node.

        :return: response object

        :Example:

        >>> ChainAPI.get_info()
        {
          "server_version": "b2eb1667",
          "head_block_num": 259590,
          "last_irreversible_block_num": 259573,
          "head_block_id": "0003f60677f3707f0704f16177bf5f007ebd45eb6efbb749f",
          "head_block_time": "2017-12-10T17:05:36",
          "head_block_producer": "initp",
          "recent_slots": "1111111111111111111111111111111111111111111111111",
          "participation_rate": "1.00000000000000000"
        }

        """
        path = '/v1/chain/get_info'
        return self.session.get(path=path)

    def get_block(self, block_id):
        """
        Get information related to a block.

        :param block_id: (str) the format must be a string of a json
        :return: response object

        :Example:

        >>> ChainAPI.get_block(block_id='{"block_num_or_id":5}')
        {
          "previous": "0000000445a9f27898383fd7de32835d5d6a978cc14ce40d9f3jb",
          "timestamp": "2017-07-18T20:16:36",
          "transaction_merkle_root": "0000000000000000000000000000000000000",
          "producer": "initf",
          "producer_changes": [ ],
          "producer_signature": "204cb94b3186c3b4a7f88be4e9db9f8af2ffdb7ef9f",
          "cycles": [ ],
          "id":"000000050c0175cbf218a70131ddc3c3fab8b6e954edef77e0bfe7c36b599b1d",
          "block_num":5,
          "ref_block_prefix":27728114
        }

        """
        path = '/v1/chain/get_block'
        return self.session.post(path=path, data=block_id)

    def get_account(self, account_name):
        """
        Get information related to an account.

        :param account_name: (str) the format must be a string of a json
        :return: response object

        :Example:

        >>> ChainAPI.get_account(account_name='{"account_name":"inita"}')
        {
          "name": "inita",
          "eos_balance": "999998.9574 EOS",
          "staked_balance": "0.0000 EOS",
          "unstaking_balance": "0.0000 EOS",
          "last_unstaking_time": "2106-02-07T06:28:15",
          "permissions": [
            {
              "name": "active",
              "parent": "owner",
              "required_auth": {
                "threshold": 1,
                "keys": [
                  {
                    "key": "EOS6MRyAjQq8ud7hVNYcfnVPJqcVpscN5SoV",
                    "weight": 1
                  }
                ],
                "accounts": []
              }
            },
            {
              "name": "owner",
              "parent": "owner",
              "required_auth": {
                "threshold": 1,
                "keys": [
                  {
                    "key": "EOS6MRyAjQq8ud7hVNYcfnVPJqcVpscN5So5CV",
                    "weight": 1
                  }
                ],
                "accounts": []
              }
            }
          ]
        }

        """
        path = '/v1/chain/get_account'
        return self.session.post(path=path, data=account_name)

    def get_code(self, account_name):
        """
        Fetch smart contract code.

        :param account_name: (str) the format must be a string of a json
        :return: response object

        :Example:

        >>> ChainAPI.get_code(account_name='{"account_name":"currency"}')
        {
          "name":"currency",
          "code_hash":"a1c8c84b4700c09c8edb83522237439e33cf01",
          "wast":"(module(type $0 (func (param i64 i64 i32) (result i32)))...truncated",
          "abi": {
          "types": [{
              "new_type_name": "account_name",
              "type": "name"
            }
          ],
          "structs": [{
              "name": "transfer",
              "base": "",
              "fields": [
                {"name":"from", "type":"account_name"},
                {"name":"to", "type":"account_name"},
                {"name":"quantity", "type":"uint64"}
              ]
            },{
              "name": "account",
              "base": "",
              "fields": [
                {"name":"key", "type":"name"},
                {"name":"balance", "type":"uint64"}
              ]
            }
          ],
          "actions": [{
              "name": "transfer",
              "type": "transfer"
            }
          ],
          "tables": [{
              "name": "account",
              "type": "account",
              "index_type": "i64",
              "key_names" : ["key"],
              "key_types" : ["name"]
            }
          ]
        }

        """
        path = '/v1/chain/get_code'
        return self.session.post(path=path, data=account_name)

    def get_table_rows(self, account_details):
        """
        Fetch smart contract data from an account.

        :param account_details: (str) account details, the format
        must be a string of a json
        :return: response object

        :Example:

        >>> ChainAPI.get_table_rows(account_details='{"scope":"inita",
                "code":"currency", "table":"account","json": true}')
        {
          "rows": [
            {
              "account": "account",
              "balance": 1000
            }
          ],
          "more": false
        }

        """
        path = '/v1/chain/get_table_rows'
        return self.session.post(path=path, data=account_details)

    def abi_json_to_bin(self, data):
        """
        Serialize json to binary hex. The resulting binary hex is usually
        used for the data field in push_transaction.

        :param data: (str) the format must be a string of a json
        :return: response object

        :Example:

        >>> ChainAPI.abi_json_to_bin(data='{"code":"currency",
                "action":"transfer", "args":{"from":"initb",
                "to":"initc", "quantity":1000}}')
        {
          "binargs": "000000008093dd74000000000094dd74e803000000000000",
          "required_scope": [],
          "required_auth": []
        }

        """
        path = '/v1/chain/abi_json_to_bin'
        return self.session.post(path=path, data=data)

    def abi_bin_to_json(self, data):
        """
        Serialize back binary hex to json.

        :param data:  (str) the format must be a string of a json
        :return:response object

        :Example:

        >>> ChainAPI.abi_bin_to_json(data='{"code":"currency",
                "action":"transfer", "binargs":"000000008093dd74000000000094dd74e803000000000000"}')
        {
          "args": {
            "from": "initb",
            "to": "initc",
            "quantity": 1000
          },
          "required_scope": [],
          "required_auth": []
        }

        """
        path = '/v1/chain/abi_bin_to_json'
        return self.session.post(path=path, data=data)

    def push_transaction(self, transaction):
        """
        This method expects a transaction in JSON format and will
        attempt to apply it to the blockchain,

        :param transaction: (str) transaction, the format must be a
        string of a json
        :return: response object

        :Example:

        >>> ChainAPI.push_transaction(transaction='{"ref_block_num":"100","ref_block_prefix":"137469861","expiration":"2017-09-25T06:28:49","scope":["initb","initc"],"actions":[{"code":"currency","type":"transfer","recipients":["initb","initc"],"authorization":[{"account":"initb","permission":"active"}],"data":"000000000041934b000000008041934be803000000000000"}],"signatures":[],"authorizations":[]}')
        {
          "args": {
            "from": "initb",
            "to": "initc",
            "quantity": 1000
          },
          "required_scope": [],
          "required_auth": []
        }

        """
        path = '/v1/chain/push_transaction'
        return self.session.post(path=path, data=transaction)

    def push_transactions(self, transactions):
        """
        This method push multiple transactions at once.

        :param transactions: (str) list of transactions, the format must be a
        string of a list
        :return: response object

        :Example:

        >>> ChainAPI.push_transactions(transactions='[{"ref_block_num":"101","ref_block_prefix":"4159312339","expiration":"2017-09-25T06:28:49","scope":["initb","initc"],"actions":[{"code":"currency","type":"transfer","recipients":["initb","initc"],"authorization":[{"account":"initb","permission":"active"}],"data":"000000000041934b000000008041934be803000000000000"}],"signatures":[],"authorizations":[]}, {"ref_block_num":"101","ref_block_prefix":"4159312339","expiration":"2017-09-25T06:28:49","scope":["inita","initc"],"actions":[{"code":"currency","type":"transfer","recipients":["inita","initc"],"authorization":[{"account":"inita","permission":"active"}],"data":"000000008040934b000000008041934be803000000000000"}],"signatures":[],"authorizations":[]}]')
        {
          "args": {
            "from": "initb",
            "to": "initc",
            "quantity": 1000
          },
          "required_scope": [],
          "required_auth": []
        }

        """
        path = '/v1/chain/push_transaction'
        return self.session.post(path=path, data=transactions)

    def get_required_keys(self, transaction_data):
        """
        Get required keys to sign a transaction from list of your keys.

        :param transaction_data: (str) transaction data with a list of keys,
        the format must be a string of a json
        :return: response object

        :Example:

        >>> ChainAPI.get_required_keys(transaction_data='{"transaction": {"ref_block_num":"100","ref_block_prefix":"137469861","expiration":"2017-09-25T06:28:49","scope":["initb","initc"],"actions":[{"code":"currency","type":"transfer","recipients":["initb","initc"],"authorization":[{"account":"initb","permission":"active"}],"data":"000000000041934b000000008041934be803000000000000"}],"signatures":[],"authorizations":[]}, "available_keys":["EOS4toFS3YXEQCkuuw1aqDLrtHim86Gz9u3hBdcBw5KNPZcursVHq","EOS7d9A3uLe6As66jzN8j44TXJUqJSK3bFjjEEqR4oTvNAB3iM9SA","EOS6MRyAjQq8ud7hVNYcfnVPJqcVpscN5So8BhtHuGYqET5GDW5CV"]}')
         {
          "required_keys": [
            "EOS6MRyAjQq8ud7hVNYcfnVPJqcVpscN5So8BhtHuGYqET5GDW5CV"
          ]
        }

        """
        path = '/v1/chain/get_required_keys'
        return self.session.post(path=path, data=transaction_data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. module:: EOSWalletApi
   :synopsis: A basic wrapper around the EOS Wallet API..
.. author:: Merouane Benthameur <merouane.benth@gmail.com>
"""


class WalletAPI:
    """wrapper for EOS Wallet API"""

    def __init__(self, connection_session):
        """
        constructor  of the WalletAPI
        :param connection_session: session request object
        """
        self.session = connection_session

    def wallet_create(self, wallet_name):
        """
        Create a new wallet with the given name.

        :param wallet_name: (str) name of the wallet to be created
        :return: response object

        :Example:

        >>> WalletAPI.wallet_create(wallet_name='"default"')
        PW5KFWYKqvt63d4iNvedfDEPVZL227D3RQ1zpVFzuUwhMAJmRAYyX

        This command will return the password that can be used to
        unlock the wallet in the future

        """
        path = '/v1/wallet/create'
        return self.session.post(path=path, data=wallet_name)

    def wallet_open(self, wallet_name):
        """
        Open an existing wallet of the given name.

        :param wallet_name: (str) name of the wallet to be opened
        :return: response object

        :Example:

        >>> WalletAPI.wallet_open(wallet_name='"default"')
        {}

        """
        path = '/v1/wallet/open'
        return self.session.post(path=path, data=wallet_name)

    def wallet_lock(self, wallet_name):
        """
        Lock a wallet of the given name

        :param wallet_name: (str) name of the wallet to be opened
        :return: response object

        :Example:

        >>> WalletAPI.wallet_lock(wallet_name='"default"')
        {}

        """
        path = '/v1/wallet/lock'
        return self.session.post(path=path, data=wallet_name)

    def wallet_lock_all(self):
        """
        Lock all wallets.

        :return: response object

        :Example:

        >>> WalletAPI.wallet_lock_all()
        {}

        """
        path = '/v1/wallet/lock_all'
        return self.session.get(path=path)

    def wallet_unlock(self, wallet_name_passord):
        """
        Unlock a wallet with the given name and password

        :param wallet_name_passord: (str) list with name and password
        of the given wallet
        :return: response object

        :Example:

        >>> WalletAPI.wallet_unlock(wallet_name_passord='["default",
               "PW5KFWYKqvt63d4iNvedfDEPVZL227D3RQ1zpVFzuUwhMAJmRAYyX"]')
        {}

        """
        path = '/v1/wallet/unlock'
        return self.session.post(path=path, data=wallet_name_passord)

    def wallet_import_key(self, wallet_name_privKey):
        """
        Import a private key to the wallet of the given name

        :param wallet_name_privKey: (str) list that contains the wallet
        name and private key
        :return: response object

        :Example:

        >>> WalletAPI.wallet_import_key(wallet_name_privKey='["default",
               "5KQwrPbwdL6PhXujxW37FSSQZ1JiwsST4cqQzDeyXtP79zkvFD3"]')

        """
        path = '/v1/wallet/import_key'
        return self.session.post(path=path, data=wallet_name_privKey)

    def wallet_list(self):
        """
        List all wallets

        :return: response object

        :Example:

        >>> WalletAPI.wallet_list()
        ["default *"]

        """
        path = '/v1/wallet/list_wallets'
        return self.session.get(path=path)

    def wallet_list_keys(self):
        """
        List all key pairs across all wallets

        :return: response object

        :Example:

        >>> WalletAPI.wallet_list_keys()
        [["EOS6MRyAjQq8ud7hVNYcfnVPJqcVpscN5So8BhtHuGYqET5GDW5CV","5KQwrPbwdL6PhXujxW37FSSQZ1JiwsST4cqQzDeyXtP79zkvFD3"]]

        """
        path = '/v1/wallet/list_keys'
        return self.session.get(path=path)

    def wallet_get_public_keys(self):
        """
        List all public keys across all wallets

        :return: response object

        :Example:

        >>> WalletAPI.wallet_get_public_keys()
        ["EOS6MRyAjQq8ud7hVNYcfnVPJqcVpscN5So8BhtHuGYqET5GDW5CV"]

        """
        path = '/v1/wallet/get_public_keys'
        return self.session.get(path=path)

    def wallet_set_timeout(self, timeout):
        """
        Set wallet auto lock timeout (in seconds)

        :param timeout: (str) timeout in seconds
        :return: response object

        :Example:

        >>> WalletAPI.wallet_set_timeout(timeout='10')
        {}

        """
        path = '/v1/wallet/set_timeout'
        return self.session.post(path=path, data=timeout)

    def wallet_sign_trx(self, transaction_data):
        """
        Sign transaction given an array of transaction, require
        public keys, and chain id

        :param transaction_data: (str) list of transaction json and  public key
        :return: response key

        :Example:

        >>> WalletAPI.wallet_sign_trx(transaction_data='[{"ref_block_num":21453,"ref_block_prefix":3165644999,"expiration":"2017-12-08T10:28:49","scope":["initb","initc"],"read_scope":[],"messages":[{"code":"currency","type":"transfer","authorization":[{"account":"initb","permission":"active"}],"data":"000000008093dd74000000000094dd74e803000000000000"}],"signatures":[]},["EOS6MRyAjQq8ud7hVNYcfnVPJqcVpscN5So8BhtHuGYqET5GDW5CV"],""]')
        {
          "ref_block_num": 21453,
          "ref_block_prefix": 3165644999,
          "expiration": "2017-12-08T10:28:49",
          "scope": [
            "initb",
            "initc"
          ],
          "read_scope": [],
          "messages": [
            {
              "code": "currency",
              "type": "transfer",
              "authorization": [
                {
                  "account": "initb",
                  "permission": "active"
                }
              ],
              "data": "000000008093dd74000000000094dd74e803000000000000"
            }
          ],
          "signatures": [
            "1f393cc5ce6a6951fb53b11812345bcf14ffd978b07be386fd639eaf440bca7dca16b14833ec661ca0703d15e55a2a599a36d55ce78c4539433f6ce8bcee0158c3"
          ]
        }

        """
        path = '/v1/wallet/sign_transaction'
        return self.session.post(path=path, data=transaction_data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 Learn more: documentation site
"""

from setuptools import setup, find_packages


with open('README.md') as f:
    readme = f.read()

with open('LICENSE.md') as f:
    license = f.read()

setup(
    name='pyeos_client',
    version='0.1.2',
    description='Wrapper for EOS RPC API',
    long_description=readme,
    author='Merouane Benthameur',
    author_email='merouane.benth@gmail.com',
    url='https://github.com/EvaCoop/pyeos_client.git',
    license=license,
    data_files=[("", ["LICENSE.md"])],
    packages=find_packages(exclude=('tests', 'docs')),
    install_requires=['requests']
)
